#!/bin/sh

#CFG_FILE="functions.def"
CFG_FILE="../../scripts/node-info.def"
VAR_FILE="variables.def"
MODULE_DIR="../modules"
BUILD_DIR="../build"
FUNC_DIR="../../functions"


# check input
if [ $# -ne 2 ]; then
    echo "Usage: $0 <config.file> <NODE_TYPE>"
    echo "Example: $0 $CFG_FILE WN"
    exit
fi 

if ! [ -f $1 ]; then
    echo "file '$1' does not exist"
    exit
fi 

# read version and list of nodes from configuration
NODES=`grep "FUNCTIONS=" ${1} | awk -F= '{print $1}' | awk -F_ '{for(i=1;i<NF-1;i++) printf $i"_";printf $(NF-1)"\n"}'`

VERSION=`grep "^VERSION" ${1} | awk -F= '{print $2}'`


found="no"
for nodename in ${NODES}; do
    if [ ${2} = "${nodename}" ]; then
        found="yes"
        break
    fi
done
if [ ${found} = "no" ]; then
    cat <<EOF

ERROR: Missing or unknown argument

Syntax: $0 $1 <NODE-TYPE>

Available calls (case sensitive):
-------------------------------------
EOF
    
    for nodename in ${NODES}; do
	echo "$0 $1 $nodename"
    done
    echo "-------------------------------------"
    exit
fi

# load variables
. ./$1

if ! [ -d ${BUILD_DIR} ]; then
    mkdir ${BUILD_DIR}
fi

# write 'wrapper' document
docnametex=`echo ${2} | sed 's/_/\\\\_/g' ` 

# if the specific Introduction Module doesn't exist, create it
if ! [ -f ${MODULE_DIR}/${2}-SpecialIntro.tex ]; then
    cat <<EOF > ${MODULE_DIR}/${2}-SpecialIntro.tex
% Special preamble for the node  ${2}
% ------------------

% If you have specific Information to produce for this
% node type please write them in this file and 
% then commit them in cvs.

% Uncomment the section header to enable the chapter
%\section{Specific Information about ${docnametex} node}

%$<$Put your text here$>$

EOF
fi

# create the node variables description module. 
# The script creates locally a description file called 
# <node-name>_vars .tex
varTeX.awk -F"+++" -v method="cre_node_desc" -v node=${2} -v funcdir="${FUNC_DIR}" -v nodeinfofile="${CFG_FILE}" ${VAR_FILE}
mv ${2}_vars.tex ${BUILD_DIR}

# _ in the node name must be escaped in latex
nodetex=`echo ${2} | sed 's/_/\\\\_/g' `

# create the node variables list module
cat << EOF > ${BUILD_DIR}/${2}_vars-head.tex
\section{Variables}
In order to set-up a ${nodetex} node, you need at least the following 
variables to be correctly configured in the site configuration file 
(site-info.def):
EOF
cat ${BUILD_DIR}/${2}_vars-head.tex ${BUILD_DIR}/${2}_vars.tex > ${BUILD_DIR}/${2}_variables.tex


cat <<EOF > ${BUILD_DIR}/${2}.tex
% ${2}.tex - ${2} configuration reference
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\def\nodeType{$docnametex}
\input{${MODULE_DIR}/AbstractIntro.tex}
\input{${MODULE_DIR}/${2}-SpecialIntro.tex}
\input{${BUILD_DIR}/${2}_variables.tex}
EOF

cat <<EOF > ${BUILD_DIR}/${2}_src.tex
\section{Source code}
EOF

FUNCTIONS=`eval echo '$'${2}_FUNCTIONS`

for func in ${FUNCTIONS}; do

    if [ -f ${FUNC_DIR}/${func} ]; then

        # create the function's source module	
	cat <<EOF > ${MODULE_DIR}/${func}_src.tex

% Source code of Function:  ${func}
% ------------------

% This file is automatically generated by the script $0
%
% Please do not edit it!
%
% If you think that the source is incorrect 
% please change the function ${FUNC_DIR}/${func} 
% and commit in CVS. This file will be generated 
% accordingly.


{\footnotesize
\begin{verbatim}
EOF
	cat ${FUNC_DIR}/${func} >> ${MODULE_DIR}/${func}_src.tex

	cat <<EOF >> ${MODULE_DIR}/${func}_src.tex
\end{verbatim}
}
% end of source code
EOF

        # create the function's variables module. 
        # The script creates locally a description file called 
        # <func-name>-vars .tex
	varTeX.awk -F"+++" -v method="cre_func_desc" -v funct="${FUNC_DIR}/${func}" ${VAR_FILE}
        mv ${func}_vars.tex ${MODULE_DIR}

    else
	echo "ERROR: File ${FUNC_DIR}/${func} does not exist"
	exit
    fi

    functex=`echo ${func} | sed 's/_/\\\\_/g' `
    # if the function module doesn't exist, create it
    if ! [ -f ${MODULE_DIR}/${func}.tex ]; then
	cat <<EOF > ${MODULE_DIR}/${func}.tex
% Specification of Function:  ${func}
% ------------------

% Possibly replace the section name with  
% an extended and more readable one
% 
% e.g. \section{Globus Gatekeeper configuration}
\section{$functex}

% If you put your name(s) there as follows
% Author(s): Surname,Name; Surname,Name \\\\
% You will have it(them) published in the authors' list
Author(s): LCG Deployment - GIS team \\\\
Email : support-lcg-manual-install@cern.ch\\\\

This chapter describes the configuration steps done by the {\em yaim}
function '{\em ${functex}}'.\\\\

% remove the following period when editing this file
A description of the configuration steps done by this function has not been 
provided yet.\\\\
If you need specific information, please get in touch with
the author.

% <insert your text here>

\subsection{Specification of function: ${functex}}
\label{${func}}

The function {\em '${functex}'} needs the following variables to be set in the 
configuration file:
\input{${MODULE_DIR}/${func}_vars.tex}

% remove this sentence if you edit this file
No further specification of the function {\em '${functex}'} has been 
provided up till now.\\\\

% <insert your text here>


The original code of the function can be found in:
{\footnotesize
\begin{verbatim}
/opt/lcg/yaim/functions/${func}
\end{verbatim}
}

The code is also reproduced in \ref{${func}_src}.

EOF
    else
	FUNCAUTHOR=`grep ^Author ${MODULE_DIR}/${func}.tex | awk -F"\\\\" '{print $1}' | awk -F":" '{print $2}'`
	AUTHORLIST="${AUTHORLIST}; ${FUNCAUTHOR}"
    fi  

    # input the module
    echo "\input{${MODULE_DIR}/${func}.tex}" >> ${BUILD_DIR}/${2}.tex

    # input the source
    echo "\subsection{$functex}" >> ${BUILD_DIR}/${2}_src.tex
    echo "\label{${func}_src}" >> ${BUILD_DIR}/${2}_src.tex
    echo "\input{${MODULE_DIR}/${func}_src.tex}" >> ${BUILD_DIR}/${2}_src.tex
done

# put the source at the end
echo "\input{${BUILD_DIR}/${2}_src.tex}" >> ${BUILD_DIR}/${2}.tex

# write the docinfo.tex file
AUTHOR=`echo "LCG Deployment - GIS team${AUTHORLIST}"|awk -F";" '{for (i=1;i<=NF;i++) {printf $i"\n"}}'|sed 's/^[ |	]*//g' | sed 's/[ |	]*$//g'| sort -uf | tr "\n" ";"`
#fi
authortex=`echo ${AUTHOR} | sed 's/_/\\\\_/g' `
cat <<EOF > ${BUILD_DIR}/docinfo.tex
\documentclass[11pt]{article}
\title{${docnametex} Generic Configuration Reference}
\def\author{${authortex}}
\def\status{ACTIVE}
\def\Subtitle{LCG - ${docnametex} - Generic Configuration Reference}
\def\Version{$VERSION}
\def\DocIdentifier{LCG-GIS-CR-${docnametex}}
\def\EDMSid{none}
\def\Section{LCG Grid Infrastructure Support}
\def\DocStatus{ACTIVE}
\def\Dissemination{PUBLIC}
\def\Abstract{Configuration steps done by the YAIM script 'configure\_${docnametex}'}
\def\date{\today}
EOF



